﻿2013-09-12 Gerhard van Rensburg (gerhardvr@email.com)

Hi,

First of all, I want to thank Spiros for his excellent work. I've tried a number of tree extensions for Yii and this is by far the best one out there. Unfortunately it contained a number of bugs, both in the core files as well as in the example that was included. I worked through the code for a number of days and because of the excellent comments in the code, I could eventually get a very good understanding of the extension. I also learned a huge amount about Yii in the process. Thanks again Spiros.

I saw on the forums that a number of people were struggling to use JsTreeBehavior. Since I had already spent a lot of time sorting out the issues I thought I'd share my efforts. 

First a disclaimer: I do not claim any credit for this great extension. All credit belongs to Spiros. I do not support this code. I do not claim that it is bug free or that it will work out of the box. It works in my environment and I know of at least one more bug (cutting a node with its children and pasting it for example), but it should get you a long way closer in getting it working.

I've also extended it for my purposes. I needed to be able to represent a single root node in the tree while I have multiple root nodes in my dataset. To accomplish this I added two new properties to JSTreeWidget. Use of this can be seen in the tree.php file where I've commented out the lines but left them on the Widget call to demonstrate their use.
1) node_ID - this is the id of the top level node that you want to show in the tree. Ex: 	'node_ID' => '111',
2) allow_drag_to_root - set to true to allow the dragging of nodes to a root position. false to prevent nodes to be dragged to a root position. Ex: 'allow_drag_to_root' => false

I do not have the time to fork the code on Github but if anybody else want to do so, feel free. I am however willing to discuss ideas to better the extension if it coincides with my own needs.

Gerhard

# JsTreeBehavior
 © 2013  [Spiros Kabasakalis](http://iws.kabasakalis.gr/)
  [The MIT License (MIT)]( http://opensource.org/licenses/MIT)
  
  ![JsTreeBehavior](https://lwuzqa.dm1.livefilestore.com/y1pZ7ydTb51qV-0JQaF5lgUl4n-DR0itJsQg2RqFRW6pSqXVeZxAi_xNPKOSFx-l4OjX5FXqCBTqYZee_PIQqlBDbq8-0iHZkBY/jstree.png?psid=1)
  
## [LIVE DEMO](http://yiilab.kabasakalis.tk/category/tree)  
  
## Overview
Tree graphic manipulation of an ActiveRecord with NestedSet Behavior using jstree plugin.
This is actually a complete rewrite of my [NestedSetAdminGUI extension](http://www.yiiframework.com/extension/nestedsetadmingui).
Instead of gii generated files for every model,I wrote a reusable behavior that provides the same functionality.I also have separated  markup from js,and the code
 is much cleaner now.Many jstree options can be set in JsTreeWidget.php file,you can add more if you wish-I did'nt include them all,there are so many.

## Setup.

- Copy the js_plugins folder in webroot folder.I don't like publishing assets.I register css and jss straight from 
  a webroot subfolder.
- Copy the widgets folder in your protected folder.It contains the JsTreeWidget class and view files.
- Copy behaviors folder containing NestedSetBehavior.php and JsTreeBehavior.php  class file in protected folder.
- Prepare the database table for your hierarchical ActiveRecord Model.
  Use the category.sql file (category table) as reference.Columns root,lft,rgt and level are essential for nestedset behavior.Apart from them you will need at least one
 column to store the label for the tree nodes,for example "name",and a primary key.You can add more columns,(like "description")
- Create the ActiveRecord class file that corresponds to category,say Category.php.Give it a nested set behavior.In Category.php :

 ~~~ php
 public function behaviors()
{
   return array(
       'NestedSetBehavior'=>array(
           'class'=>'application.behaviors.NestedSetBehavior',
           'leftAttribute'=>'lft',
           'rightAttribute'=>'rgt',
           'levelAttribute'=>'level',
           'hasManyRoots'=>true
           ),
   );
}
 ~~~
 
- Copy  BaseController.php file to components folder.This controller makes possible for controllers that extend from it to turn behavior inherited functions
 into actions.
- Create a controller in protected/controllers folder  that extends from BaseController.php,CategoryController  and give it a JsTreeBehavior.In CategoryController:
 ~~~ php
  public function behaviors()
     {
         return array(
             'jsTreeBehavior' => array('class' => 'application.behaviors.JsTreeBehavior',
                 'modelClassName' => 'Category',
                 'form_alias_path' => 'application.views.category._form',
                 'view_alias_path' => 'application.views.category.view',
                     'label_property' => 'name',
                     'rel_property' => 'name'
             )
         );
     }
 ~~~
 - modelClassName is the class name of the ActiveRecord,form_alias_path is alias path of the form view file used
 to update and create nodes in the tree with fancybox pop ups.view_alias_path is the view file path alias used to show details for a record,(property values).
 You can easily modify the example files provides to suit your own model.You will only need to change/add input fields.
 label_property is the model property that will show as label in the tree nodes.
 rel_property is the model property that will be used as value for the rel property of the li tags in the tree markup.

- Last, add an action actionTree in Category Controller that renders a tree.php file.In tree.php file render the JsTreeWidget like so
 
 ~~~ php
 $this->widget('application.widgets.JsTreeWidget',
                       array('modelClassName' => 'Category',
                                 'jstree_container_ID' => 'Category-wrapper',//jstree will be rendered in this div.id of your choice.
                                 'themes' => array('theme' => 'default', 'dots' => true, 'icons' => true),
                                 'plugins' => array('themes', 'html_data', 'contextmenu', 'crrm', 'dnd', 'cookies', 'ui')
                                             ));
 ?>
  ~~~
  
 Fot themes and plugins options see [jtree documentation](http://www.jstree.com/documentation)

- Make sure you include jquery.Either uncomment the relevant line in JsTreeWidget or register it somewhwere else in your code.
- Example files use bootstrap styled markup.Uncomment the relevant line that registers bootstrap.css in JsTreeWidget
   if it's not already registered somewhere else in your application.
- Example files provided.
- To manipulate the tree,navigate to category/tree.Right click on nodes to see available operations.Drag and drop nodes,delete,create,update etc.


    ##Resources
- [jsTree ](http://www.jstree.com/)
- [NestedSetBehavior](http://www.yiiframework.com/extension/nestedsetbehavior)
- [Fancybox](http://www.fancyapps.com/fancybox/)
- [Noty](http://needim.github.com/noty/)
- [Bootstrap](http://twitter.github.com/bootstrap/)
- [JQueryForm Plugin]( http://malsup.com/jquery/form/)
- [spin.js](http://fgnass.github.com/spin.js/)






